@@ -1,22 +1,14 @@
 """
-SQLObject
----------
+SQLObject 0.9.2
+---------------
 
 :author: Ian Bicking <ianb@colorstudy.com>
 
 SQLObject is a object-relational mapper.  See SQLObject.html or
 SQLObject.txt for more.
 
-Modified by:
-
-* Daniel Savard, Xsoli Inc <sqlobject xsoli.com> 7 Feb 2004
-  Added support for simple table inheritance.
-
-* Oleg Broytmann, SIA "ANK" <phd@phd.pp.ru> 3 Feb 2005
-  Split inheritance support into a number of separate modules and classes -
-  InheritableSQLObject at al.
-
-* And others...
+With the help by Oleg Broytmann <phd@phd.pp.ru> and many other contributors.
+See Authors.txt.
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU Lesser General Public License as
@@ -220,7 +212,7 @@ class sqlmeta(object):
     __metaclass__ = declarative.DeclarativeMeta
 
     # These attributes shouldn't be shared with superclasses:
-    _unshared_attributes = ['table', 'idName', 'columns', 'childName']
+    _unshared_attributes = ['table', 'columns', 'childName']
 
     # These are internal bookkeeping attributes; the class-level
     # definition is a default for the instances, instances will
@@ -493,7 +485,7 @@ class sqlmeta(object):
 
         if changeSchema:
             conn = connection or soClass._connection
-            conn.delColumn(sqlmeta.table, column)
+            conn.delColumn(sqlmeta, column)
 
         if soClass._SO_finishedClassCreation:
             unmakeProperties(soClass)
@@ -655,9 +647,9 @@ class _sqlmeta_attr(object):
 warnings_level = 1
 exception_level = None
 # Current levels:
-#  1) Actively deprecated in version after 0.6.1 (0.7?); removed after
-#  2) Deprecated after 1 (0.8?)
-#  3) Deprecated after 2 (0.9?)
+#  1) Actively deprecated in version after 0.9; removed after
+#  2) Deprecated after 1 (0.9?)
+#  3) Deprecated after 2 (0.10?)
 
 def deprecated(message, level=1, stacklevel=2):
     if exception_level is not None and exception_level <= level:
@@ -665,6 +657,9 @@ def deprecated(message, level=1, stacklevel=2):
     if warnings_level is not None and warnings_level <= level:
         warnings.warn(message, DeprecationWarning, stacklevel=stacklevel)
 
+if sys.version_info[:3] < (2, 3, 0):
+    deprecated("Support for Python 2.2 has been declared obsolete and will be removed in the next release of SQLObject")
+
 def setDeprecationLevel(warning=1, exception=None):
     """
     Set the deprecation level for SQLObject.  Low levels are more
@@ -676,8 +671,8 @@ def setDeprecationLevel(warning=1, exception=None):
 
     The levels currently mean:
 
-      1) Deprecated in current version (0.7).  Will be removed in next
-         version (0.8)
+      1) Deprecated in current version (0.9).  Will be removed in next
+         version (0.10)
 
       2) Planned to deprecate in next version, remove later.
 
@@ -717,8 +712,8 @@ class SQLObject(object):
     _parent = None # A reference to the parent instance
     childName = None # Children name (to be able to get a subclass)
     # moved to sqlmeta in 0.8:
-    _parentClass = _sqlmeta_attr('parentClass', 2)
-    _childClasses = _sqlmeta_attr('childClasses', 2)
+    _parentClass = _sqlmeta_attr('parentClass', 1)
+    _childClasses = _sqlmeta_attr('childClasses', 1)
 
     # The law of Demeter: the class should not call another classes by name
     SelectResultsClass = SelectResults
@@ -753,24 +748,6 @@ class SQLObject(object):
 
         cls._SO_finishedClassCreation = False
 
-        if '_columns' in new_attrs and not is_base:
-            deprecated(
-                'The _columns attribute you gave (%r) is deprecated; '
-                'columns should be added as class attributes'
-                % new_attrs['_columns'], level=1)
-            for column in new_attrs['_columns']:
-                if isinstance(column, str):
-                    column = col.Col(column)
-                implicitColumns.append(column)
-
-        if '_joins' in new_attrs and not is_base:
-            deprecated(
-                'The _joins attribute you gave (%r) is deprecated; '
-                'joins should be added as class attributes'
-                % new_attrs['_joins'], level=1)
-            for j in new_attrs['_joins']:
-                implicitJoins.append(j)
-
         ######################################################
         # Set some attributes to their defaults, if necessary.
         # First we get the connection:
@@ -784,10 +761,6 @@ class SQLObject(object):
         if connection and not hasattr(cls, '_connection'):
             cls.setConnection(connection)
 
-        # Now the class is in an essentially OK-state, so we can
-        # set up any magic attributes:
-        declarative.setup_attributes(cls, new_attrs)
-
         # We have to check if there are columns in the inherited
         # _columns where the attribute has been set to None in this
         # class.  If so, then we need to remove that column from
@@ -803,6 +776,10 @@ class SQLObject(object):
         for column in implicitColumns:
             cls.sqlmeta.addColumn(column)
 
+        # Now the class is in an essentially OK-state, so we can
+        # set up any magic attributes:
+        declarative.setup_attributes(cls, new_attrs)
+
         if cls.sqlmeta.fromDatabase:
             cls.sqlmeta.addColumnsFromDatabase()
 
@@ -825,23 +802,21 @@ class SQLObject(object):
 
         classregistry.registry(cls.sqlmeta.registry).addClass(cls)
 
-    _style = _sqlmeta_attr('style', 2)
-    _table = _sqlmeta_attr('table', 2)
-    _idName = _sqlmeta_attr('idName', 2)
-    _lazyUpdate = _sqlmeta_attr('lazyUpdate', 2)
-    _defaultOrder = _sqlmeta_attr('defaultOrder', 2)
-    _cacheValues = _sqlmeta_attr('cacheValues', 2)
-    _registry = _sqlmeta_attr('registry', 2)
-    _idType = _sqlmeta_attr('idType', 2)
-    _fromDatabase = _sqlmeta_attr('fromDatabase', 2)
+    _style = _sqlmeta_attr('style', 1)
+    _table = _sqlmeta_attr('table', 1)
+    _idName = _sqlmeta_attr('idName', 1)
+    _lazyUpdate = _sqlmeta_attr('lazyUpdate', 1)
+    _defaultOrder = _sqlmeta_attr('defaultOrder', 1)
+    _cacheValues = _sqlmeta_attr('cacheValues', 1)
+    _registry = _sqlmeta_attr('registry', 1)
+    _idType = _sqlmeta_attr('idType', 1)
+    _fromDatabase = _sqlmeta_attr('fromDatabase', 1)
     _expired = _sqlmeta_attr('expired', 2)
-    _columns = _sqlmeta_attr('columnList', 1)
-    _columnDict = _sqlmeta_attr('columns', 1)
-    addColumn = _sqlmeta_attr('addColumn', 2)
-    delColumn = _sqlmeta_attr('delColumn', 2)
-    addJoin = _sqlmeta_attr('addJoin', 2)
-    delJoin = _sqlmeta_attr('delJoin', 2)
-    addIndex = _sqlmeta_attr('addIndex', 2)
+    addColumn = _sqlmeta_attr('addColumn', 1)
+    delColumn = _sqlmeta_attr('delColumn', 1)
+    addJoin = _sqlmeta_attr('addJoin', 1)
+    delJoin = _sqlmeta_attr('delJoin', 1)
+    addIndex = _sqlmeta_attr('addIndex', 1)
 
     # @classmethod
     def _SO_setupSqlmeta(cls, new_attrs, is_base):
@@ -901,7 +876,7 @@ class SQLObject(object):
                 new_name = attr[1:]
                 deprecated("%r is deprecated; please set the %r "
                            "attribute in sqlmeta instead" %
-                           (attr, new_name), level=2,
+                           (attr, new_name), level=1,
                            stacklevel=5)
                 setattr(cls.sqlmeta, new_name, new_attrs[attr])
                 delattr(cls, attr)
@@ -909,7 +884,7 @@ class SQLObject(object):
             if new_attrs.has_key(attr):
                 deprecated("%r is deprecated and read-only; please do "
                            "not use it in your classes until it is fully "
-                           "deprecated" % attr, level=3, stacklevel=5)
+                           "deprecated" % attr, level=2, stacklevel=5)
 
     _SO_cleanDeprecatedAttrs = classmethod(_SO_cleanDeprecatedAttrs)
 
@@ -1058,7 +1033,7 @@ class SQLObject(object):
         # the parts are set.  So we just keep them in a
         # dictionary until later:
         d = {name: value}
-        if not self.sqlmeta._creating:
+        if not self.sqlmeta._creating and not getattr(self.sqlmeta, "row_update_sig_suppress", False):
             self.sqlmeta.send(events.RowUpdateSignal, self, d)
         if len(d) != 1 or name not in d:
             # Already called RowUpdateSignal, don't call it again
@@ -1252,7 +1227,8 @@ class SQLObject(object):
                 default = column.default
 
                 # If we don't get it, it's an error:
-                if default is NoDefault:
+                # If we specified an SQL DEFAULT, then we should use that
+                if default is NoDefault and column.defaultSQL is not None:
                     raise TypeError, "%s() did not get expected keyword argument %s" % (self.__class__.__name__, column.name)
                 # Otherwise we put it in as though they did pass
                 # that keyword:
@@ -1365,10 +1341,16 @@ class SQLObject(object):
 
     selectBy = classmethod(selectBy)
 
+    def tableExists(cls, connection=None):
+        conn = connection or cls._connection
+        return conn.tableExists(cls.sqlmeta.table)
+
+    tableExists = classmethod(tableExists)
+
     def dropTable(cls, ifExists=False, dropJoinTables=True, cascade=False,
                   connection=None):
         conn = connection or cls._connection
-        if ifExists and not conn.tableExists(cls.sqlmeta.table):
+        if ifExists and not cls.tableExists(connection=conn):
             return
         extra_sql = []
         post_funcs = []
@@ -1387,7 +1369,7 @@ class SQLObject(object):
                     createIndexes=True, applyConstraints=True,
                     connection=None):
         conn = connection or cls._connection
-        if ifNotExists and conn.tableExists(cls.sqlmeta.table):
+        if ifNotExists and cls.tableExists(connection=conn):
             return
         extra_sql = []
         post_funcs = []
@@ -1574,6 +1556,19 @@ class SQLObject(object):
 
     delete = classmethod(delete)
 
+    def deleteMany(cls, where=NoDefault, connection=None):
+        conn = connection or cls._connection
+        conn.query(conn.sqlrepr(sqlbuilder.Delete(cls.sqlmeta.table, where)))
+
+    deleteMany = classmethod(deleteMany)
+
+    def deleteBy(cls, connection=None, **kw):
+        conn = connection or cls._connection
+        conn.query(conn.sqlrepr(sqlbuilder.Delete(cls.sqlmeta.table,
+            conn._SO_columnClause(cls, kw))))
+
+    deleteBy = classmethod(deleteBy)
+
     def __repr__(self):
         if not hasattr(self, 'id'):
             # Object initialization not finished.  No attributes can be read.
@@ -1612,6 +1607,7 @@ class SQLObject(object):
         cls._connection = value
     setConnection = classmethod(setConnection)
 
+
 def capitalize(name):
     return name[0].capitalize() + name[1:]
 
